# -*- coding: utf-8 -*-

# =============================================================
# Author: http://sefikail.cz
# =============================================================

import os
import errno
from platform import system
from subprocess import call
from random import choice
from shutil import rmtree
from tempfile import gettempdir


TMP_MAX = 10000  # Try again, max number


def generate_random_chain(length=12):
	# @formatter:off (pycharm - no formatting)
	characters = {
		'letters': 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ',
		'digits': '0123456789'
	}
	# @formatter:on (pycharm - no formatting)

	all_chars = ''.join(str(x) for x in characters.values())  # All the characters in one variable
	rand_chars = ''.join(choice(all_chars) for x in range(length))  # Generate random characters
	return rand_chars


def TemporaryDirectory(suffix='', prefix='', dir=None, delete=True):
	if not dir or dir is None:
		dir = gettempdir()

	tempdir = os.path.join(dir, prefix + generate_random_chain() + suffix)
	for i in range(TMP_MAX, 0, -1):
		try:
			os.mkdir(tempdir)
		except OSError as e:
			if e.errno == errno.EEXIST:  # If folder exists, try again.
				tempdir = os.path.join(dir, prefix + generate_random_chain() + suffix)
			else:
				raise e
		else:
			return TemporaryDirectoryWrapper(tempdir, delete)
	raise Exception('Cannot create temporary directory "%s".' % tempdir)


class TemporaryDirectoryWrapper:
	def __init__(self, tempdir, delete=True):
		self.tempdir = tempdir
		self.delete = delete
		self.rmtemp_called = False
		#
		self.os_platform = system()
		self.rm_error_access = None

		if not os.path.exists(tempdir):
			raise Exception('Cannot find temporary directory "%s".' % tempdir)

	def __enter__(self):
		return self

	def __exit__(self, exc_type, exc_val, exc_tb):
		self.rmtemp()

	def __del__(self):
		self.rmtemp()

	def rmtemp(self):
		if not self.rmtemp_called:
			self.rmtemp_called = True
			if self.delete and os.path.exists(self.tempdir):
				try:
					rmtree(self.tempdir)
				except Exception as e:
					if e.errno == errno.EACCES:
						self.rm_error_access = str(e)
						if self.os_platform == "Windows":
							call(['cmd', '/c', 'rmdir', '/s', '/q', self.tempdir], shell=False)
					else:
						raise e
				finally:
					if os.path.exists(self.tempdir):
						raise Exception('Cannot remove temporary directory "%s", "%s"' % (self.tempdir, self.rm_error_access))

	@property
	def name(self):
		return self.tempdir



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import sptempdir


print("========================================================")
print("Example 1:")
print("---------")

with sptempdir.TemporaryDirectory(prefix="prefbegin_", suffix="_suffend") as temp:
	print('temp.name:', temp.name)  # retrieve the name temporary directory
	print('Inside:', os.path.exists(temp.name))

print('Outside:', os.path.exists(temp.name))

#
print("========================================================")
print("Example 2:")
print("---------")

temp = sptempdir.TemporaryDirectory()
print('temp.name:', temp.name)  # retrieve the name temporary directory
print('Tempdir exists:', os.path.exists(temp.name))

temp.rmtemp()  # manually remove temporary directory
print('Tempdir exists:', os.path.exists(temp.name))

#
print("========================================================")
print("Example 3:")
print("---------")

temp = sptempdir.TemporaryDirectory(delete=False)
print('temp.name:', temp.name)  # retrieve the name temporary directory
print('Tempdir exists:', os.path.exists(temp.name))

temp.rmtemp()  # manually remove temporary directory
print('Tempdir exists:', os.path.exists(temp.name))

#
print("========================================================")
print("Example 4:")
print("---------")

temp = sptempdir.TemporaryDirectory(dir="/home/user/Desktop")
print(temp.name)  # retrieve the name temporary directory



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


# Upload pypi:
# python setup.py sdist --formats=gztar upload

# @formatter:off (pycharm - no formatting)
try:
	import pypandoc
	long_description_text = pypandoc.convert('README.md', 'rst')
except Exception as e:
	long_description_text = ''


setup(
	name='sptempdir',

	# https://packaging.python.org/en/latest/distributing.html#version
	version='0.1.4',

	keywords=['tempdir', 'sptempdir', 'temporary directory'],
	description='This module generates temporary directories',
	long_description=long_description_text,

	# The project homepage
	url='https://github.com/sefikail/sptempdir/',

	# Author details
	author='sefikail',
	author_email='aleskrejcicz@gmail.com',

	packages=find_packages(exclude=['docs', 'tests', 'examples']),
	include_package_data=True,

	platforms='any',
	# https://packaging.python.org/en/latest/distributing.html#classifiers
	classifiers=[
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 2.6',
		'Programming Language :: Python :: 2.7',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.2',
		'Programming Language :: Python :: 3.3',
		'Programming Language :: Python :: 3.4',
	],

	# License
	license="See: https://creativecommons.org/licenses/by/3.0/",
)
# @formatter:on (pycharm - no formatting)
